/*-------------------------------------------------------------------------------------------*\
|                                                                                             |
|    Copyright (C) 1999 by Punk Productions Electronic Entertainment. All Rights Reserved.    | 
|                                                                                             |
|    Purpose.....: Shows how to load JPEG files into a DDsurface                              |
|    Programmer..: Nikolaus Brennig, 19, (Nikolaus.Brennig@nol.at)                            |
|	 Contact.....: virtalnik@nol.at || ICQ: 20345533										  |
|	 Internet....: http://cust.nol.at/ppee													  |
|    Help........: Joine the PPEE-GamesDev MailingList			                              |
|    Day created.: February 17th, 1999                                                        |
|    Update......: March, 30th, 1999                                                          |
|				   Can't remember anymore what i've changed :)								  |
|				   October, 3rd, 1999:														  |
|				   Completely rewritten, Faster Loading/Saving, C++ Interface                 |
|				   now the original JPEGLib 6.2 is used, also Support for                     |
|				   loading PPM/PGM files. Requires at least DirectX7                          |
|                                                                                             |
\*-------------------------------------------------------------------------------------------*/
#define WIN32_LEAN_AND_MEAN
#include "Main.h"
#include "JpgLoader.h"


//---------------------------------------------------------------------------------------------
// Local ProtoTypes:
//---------------------------------------------------------------------------------------------
LPDIRECTDRAWSURFACE7 LoadJPG( LPDIRECTDRAW7 lpDD, LPSTR Filename );
LPDIRECTDRAWSURFACE7 LoadPPM( LPDIRECTDRAW7 lpDD, LPSTR Filename );



//---------------------------------------------------------------------------------------------
// A small Wrapper for easier ImageLoading...
// Note: Make sure the "Filename" extension is for JPEGFormat *.jpg, for PPMFormat *.ppm
// and for PGMFormat *.pgm.
//---------------------------------------------------------------------------------------------
bool GetImage( LPSTR Filename, LPDIRECTDRAWSURFACE7 *surf )
{
    int len = lstrlen( Filename );

	// Load the correct FileType from the extension...
    if( lstrcmpi( Filename+(len-3), "jpg" ) == 0 )
		*surf = LoadJPG( lpDD, Filename );
    else // PGM/PPM
		*surf = LoadPPM( lpDD, Filename );

	// Now do an automatical errorcheck...
    if( *surf == NULL ) 
	{ 
		CHAR *errstr = new CHAR[512];
		sprintf( errstr, "Couldn't load \"%s\"! Check the Directory/Filename.", Filename ); 
		Error( errstr ); 
		delete errstr;
		return false;
	}

	// Yahoo...
	return true;
}


//---------------------------------------------------------------------------------------------
// Write a JPEG File...
//---------------------------------------------------------------------------------------------
VOID SaveJPG( LPSTR Filename, LPDIRECTDRAWSURFACE7 lpSurf, INT Quality, BOOL Color )
{
	PBITMAPINFO		pbmi;
	BITMAP			bmp; 
	HBITMAP			hBmp;
	HDC				hdc;
	HDC				MemDC;
	DDSURFACEDESC2	ddsd;
	INT				W, H;
	BYTE			*tmpBuffer = NULL;
	BYTE			*destbuffer = NULL;


	// First get the size of the surface...
	ZeroMemory( &ddsd, sizeof(ddsd) );
	ddsd.dwSize  = sizeof(ddsd);
	ddsd.dwFlags = DDSD_HEIGHT | DDSD_WIDTH;
	lpSurf->GetSurfaceDesc(&ddsd);
	W = ddsd.dwWidth; H = ddsd.dwHeight;

	// Then get the content of the surface into a DC...
	if( SUCCEEDED(lpSurf->GetDC(&hdc)) )
	{
		MemDC = CreateCompatibleDC( NULL );
		hBmp = CreateCompatibleBitmap( hdc, W, H );
		SelectObject( MemDC, hBmp );
		BitBlt( MemDC, 0, 0, W, H, hdc, 0, 0, SRCCOPY );
		lpSurf->ReleaseDC(hdc);
	} 
	else return;

	// Get the important WidthBytes...
	INT			WidthBytes = W*3;
	if(WidthBytes & 0x003) WidthBytes = (WidthBytes | 3) + 1;

	// Bitmapstruct init...
	GetObject( hBmp, sizeof(BITMAP), (LPSTR)&bmp );
	pbmi = (PBITMAPINFO) LocalAlloc(LPTR, sizeof(BITMAPINFOHEADER));
   
	// Initialize the fields in the BITMAPINFO structure.
	pbmi->bmiHeader.biSize                        = sizeof(BITMAPINFOHEADER); 
	pbmi->bmiHeader.biWidth                       = bmp.bmWidth; 
	pbmi->bmiHeader.biHeight                      = bmp.bmHeight; 
	pbmi->bmiHeader.biPlanes                      = bmp.bmPlanes; 
	pbmi->bmiHeader.biBitCount                    = 24; 
	pbmi->bmiHeader.biCompression                 = BI_RGB; 
	pbmi->bmiHeader.biSizeImage                   = WidthBytes * H; 
	pbmi->bmiHeader.biClrImportant                = 0; 

	// Alloc Memory...
	destbuffer = new BYTE[pbmi->bmiHeader.biSizeImage];

	// Get the bits from the DC to save it to disk...
	GetDIBits( MemDC, hBmp, 0, (unsigned int) bmp.bmHeight, destbuffer, pbmi, 0 );

	// un-DWORD-align
	tmpBuffer = CJPEG::MakeRGBAligned( destbuffer, bmp.bmWidth, WidthBytes, bmp.bmHeight );

	// vertical flip
	CJPEG::FlipVertically( tmpBuffer, WidthBytes, bmp.bmHeight );

	// reverse BGR
	CJPEG::RGBFromBGR( tmpBuffer, bmp.bmWidth, bmp.bmHeight );

	// write it
	if( !CJPEG::SaveJPEG( Filename, tmpBuffer, bmp.bmWidth, bmp.bmHeight, Color, Quality ) )
		Error( "Write Error during JPEGWrite!" );

	// Free the stuff...
	delete tmpBuffer;
	delete destbuffer;
	DeleteObject( hBmp );
	DeleteDC( MemDC );
	LocalFree((PBITMAPINFO)pbmi);

	// Yeah!
	return;
}


//---------------------------------------------------------------------------------------------
// Load a JPG into a DC...
//---------------------------------------------------------------------------------------------
LPDIRECTDRAWSURFACE7 LoadJPG( LPDIRECTDRAW7 lpDD, LPSTR Filename )
{
	UINT				 height;
	UINT				 width;
	BYTE				 *dataBuf;
	PBITMAPINFO			 bmpi;
	HDC					 hDC;
	DDSURFACEDESC2		 ddsd;
	LPDIRECTDRAWSURFACE7 surf;
	INT					 W, H, Bits;

	// Read JPEGData into a buffer...
	dataBuf = CJPEG::LoadJPEG( Filename, &width, &height, &Bits );
	if( dataBuf == NULL ) return NULL;

	// Transfer Variables...
	W = width; H = height; Bits = 24;
	INT lineWidth = W * 3;
    if(lineWidth & 0x003) lineWidth = (lineWidth | 3) + 1;

	// Create the DirectDraw Surface...
	ZeroMemory(&ddsd, sizeof(ddsd));
	ddsd.dwSize          = sizeof(ddsd);
	ddsd.dwFlags         = DDSD_CAPS | DDSD_WIDTH | DDSD_HEIGHT;
	ddsd.ddsCaps.dwCaps  = DDSCAPS_OFFSCREENPLAIN;
	ddsd.dwWidth         = W;
	ddsd.dwHeight        = H;
	if( FAILED(lpDD->CreateSurface( &ddsd, &surf, NULL )) ) return NULL;
       
	// Init the Bitmapstruct...
    bmpi = (PBITMAPINFO) LocalAlloc(LPTR, sizeof(BITMAPINFOHEADER) + (256 * sizeof(RGBQUAD)));
	bmpi->bmiHeader.biSize				= sizeof(BITMAPINFOHEADER);
	bmpi->bmiHeader.biWidth				= W;
	bmpi->bmiHeader.biHeight			= H;
	bmpi->bmiHeader.biPlanes			= 1;
	bmpi->bmiHeader.biCompression       = BI_RGB; 
	bmpi->bmiHeader.biBitCount			= Bits;
	bmpi->bmiHeader.biSizeImage			= 0;
	bmpi->bmiHeader.biClrUsed			= 0; 
	bmpi->bmiHeader.biClrImportant		= 0; 

	// Blt the stuff now...
	if( SUCCEEDED(surf->GetDC(&hDC)) )
	{
		StretchDIBits( hDC, 0, 0, W, H, 0, 0, W, H, dataBuf, bmpi, 0, SRCCOPY );
		surf->ReleaseDC(hDC);
	}

	// free the stuff...
	delete dataBuf;
	LocalFree((PBITMAPINFO)bmpi);

	// Yahoo...
	return surf;
}


//---------------------------------------------------------------------------------------------
// Load a PPM/PGM into a DC...
//---------------------------------------------------------------------------------------------
LPDIRECTDRAWSURFACE7 LoadPPM( LPDIRECTDRAW7 lpDD, LPSTR Filename )
{
	UINT				 height;
	UINT				 width;
	BYTE				 *dataBuf;
	PBITMAPINFO			 bmpi;
	HDC					 hDC;
	DDSURFACEDESC2		 ddsd;
	LPDIRECTDRAWSURFACE7 surf;
	INT					 W, H, Bits;

	// Read the PPM/PGM Data into our Buffer...
	dataBuf = CJPEG::LoadPPM( Filename, &width, &height, &Bits );
	if( dataBuf == NULL ) return NULL;

	// Transfer Variables...
	W = width; H = height; Bits = 24;
	INT lineWidth = W * 3;
    if(lineWidth & 0x003) lineWidth = (lineWidth | 3) + 1;

	// Create the DirectDraw Surface...
	ZeroMemory(&ddsd, sizeof(ddsd));
	ddsd.dwSize          = sizeof(ddsd);
	ddsd.dwFlags         = DDSD_CAPS | DDSD_WIDTH | DDSD_HEIGHT;
	ddsd.ddsCaps.dwCaps  = DDSCAPS_OFFSCREENPLAIN;
	ddsd.dwWidth         = W;
	ddsd.dwHeight        = H;
	if( FAILED(lpDD->CreateSurface( &ddsd, &surf, NULL )) ) return NULL;
       
	// Init the Bitmapstruct...
    bmpi = (PBITMAPINFO) LocalAlloc(LPTR, sizeof(BITMAPINFOHEADER) + (256 * sizeof(RGBQUAD)));
	bmpi->bmiHeader.biSize				= sizeof(BITMAPINFOHEADER);
	bmpi->bmiHeader.biWidth				= W;
	bmpi->bmiHeader.biHeight			= H;
	bmpi->bmiHeader.biPlanes			= 1;
	bmpi->bmiHeader.biCompression       = BI_RGB; 
	bmpi->bmiHeader.biBitCount			= Bits;
	bmpi->bmiHeader.biSizeImage			= 0;
	bmpi->bmiHeader.biClrUsed			= 0; 
	bmpi->bmiHeader.biClrImportant		= 0; 

	// Blt the stuff now...
	if( SUCCEEDED(surf->GetDC(&hDC)) )
	{
		StretchDIBits( hDC, 0, 0, W, H, 0, 0, W, H, dataBuf, bmpi, 0, SRCCOPY );
		surf->ReleaseDC(hDC);
	}

	// free the stuff...
	delete dataBuf;
	LocalFree((PBITMAPINFO)bmpi);

	// Yahoo...
	return surf;
}
