/*-------------------------------------------------------------------------------------------*\
|                                                                                             |
|    Copyright (C) 1999 by Punk Productions Electronic Entertainment. All Rights Reserved.    | 
|                                                                                             |
|    Purpose.....: Shows how to load JPEG files into a DDsurface                              |
|    Programmer..: Nikolaus Brennig, 19, (Nikolaus.Brennig@nol.at)                            |
|	 Contact.....: virtalnik@nol.at || ICQ: 20345533										  |
|	 Internet....: http://cust.nol.at/ppee													  |
|    Help........: Joine the PPEE-GamesDev MailingList			                              |
|    Day created.: February 17th, 1999                                                        |
|    Update......: March, 30th, 1999                                                          |
|				   Can't remember anymore what i've changed :)								  |
|				   October, 3rd, 1999:														  |
|				   Completely rewritten, Faster Loading/Saving, C++ Interface                 |
|				   now the original JPEGLib 6.2 is used, also Support for                     |
|				   loading PPM/PGM files. Requires at least DirectX7                          |
|                                                                                             |
\*-------------------------------------------------------------------------------------------*/

#define WIN32_LEAN_AND_MEAN
#include "Main.h"


//---------------------------------------------------------------------------------------------
// Libs we need to link...
//---------------------------------------------------------------------------------------------
#pragma comment(lib, "ddraw.lib")
#pragma comment(lib, "dxguid.lib")



//---------------------------------------------------------------------------------------------
// Macro for releasing Objects...
//---------------------------------------------------------------------------------------------
#define RELEASE(x) if( x != NULL ) { x->Release(); x = NULL; }



//---------------------------------------------------------------------------------------------
// Variables...
//---------------------------------------------------------------------------------------------
LPDIRECTDRAW7           lpDD;            // DirectDraw object...
LPDIRECTDRAWSURFACE7    lpDDSPrimary;    // DirectDraw primary surface...
LPDIRECTDRAWSURFACE7    lpDDSBack;       // DirectDraw backbuffer...
LPDIRECTDRAWSURFACE7    lpDDSJPGNormal;
LPDIRECTDRAWSURFACE7    lpDDSJPGProgressive;
LPDIRECTDRAWSURFACE7    lpDDSJPGGrayScale;
LPDIRECTDRAWSURFACE7    lpDDSPPMNormal;
LPDIRECTDRAWSURFACE7    lpDDSPGMNormal;
HWND                    hwnd;
HDC                     hdc;
int                     bActive; 
int                     ProgramState;
int                     TextX;
bool                    TextLeft;


//---------------------------------------------------------------------------------------------
// Paints a surface onto the Backbuffer...
//---------------------------------------------------------------------------------------------
VOID bltObject( int x, int y, LPDIRECTDRAWSURFACE7 surf, LPRECT src, DWORD flags )
{
	lpDDSBack->BltFast( x, y, surf, src, flags );
}


//---------------------------------------------------------------------------------------------
// Exchanges the FrontBufferPointer with the next BackbufferPointer...
//---------------------------------------------------------------------------------------------
VOID Flipscreen()
{
	lpDDSPrimary->Flip( NULL, DDFLIP_WAIT );
}


//---------------------------------------------------------------------------------------------
// paints the whole screen black. Nothing more.
//---------------------------------------------------------------------------------------------
VOID Blackscreen()
{
	DDBLTFX             ddbltfx;
	HRESULT             ddrval;

	ddbltfx.dwSize = sizeof( ddbltfx );
	ddbltfx.dwFillColor = 0;
 
	while( 1 )
	{
		ddrval = lpDDSBack->Blt( NULL, NULL, NULL, DDBLT_COLORFILL, &ddbltfx );
		if( ddrval == DD_OK ) break;
		if( ddrval != DDERR_WASSTILLDRAWING ) return;
	}
}


//---------------------------------------------------------------------------------------------
// This function paints the Backgroundbitmap...
//---------------------------------------------------------------------------------------------
VOID PaintBackground()
{
	RECT rcSrc = { 0, 0, 256, 192 }; // <-- Dimensions...
	bltObject( 384, 0, lpDDSJPGGrayScale, &rcSrc, DDBLTFAST_NOCOLORKEY );
	bltObject( 0, 0, lpDDSJPGProgressive, &rcSrc, DDBLTFAST_NOCOLORKEY );
	bltObject( 0, 480-192, lpDDSPPMNormal, &rcSrc, DDBLTFAST_NOCOLORKEY );
	bltObject( 384, 480-192, lpDDSPGMNormal, &rcSrc, DDBLTFAST_NOCOLORKEY );
	bltObject( 196, 144, lpDDSJPGNormal, &rcSrc, DDBLTFAST_NOCOLORKEY );
}


//---------------------------------------------------------------------------------------------
// Draw some text with GDI...
//---------------------------------------------------------------------------------------------
VOID Text( int x, int y, char *words, COLORREF Color )
{
	SetTextColor( hdc, Color );
	TextOut( hdc, x, y, words, lstrlen(words) );
}


//---------------------------------------------------------------------------------------------
// handles the whole app after intialization is done. What to paint, to do....
//---------------------------------------------------------------------------------------------
VOID HeartBeat( void )
{
	switch( ProgramState )
    {
		case PS_INITAPP: 
			TextX = 240;
			TextLeft = TRUE;
			ProgramState = PS_ACTIVE;
			return;

		case PS_ACTIVE:
			// Erase the Screen...
			Blackscreen();
			// Do the Background...
			PaintBackground();
			// Paint our great message and move it too...
			if( TextLeft ) TextX -= 5; else TextX += 5;
			if( TextX > 475 || TextX < 15) TextLeft = !TextLeft; 
			if( SUCCEEDED(lpDDSBack->GetDC(&hdc)) )
			{
				SetBkMode( hdc, TRANSPARENT );
				SetBkColor( hdc, RGB( 0, 0, 0) );
				Text( TextX, 250, "JPEGLoad Sample", RGB(255,255,0));
				Text( 230, 350, "Esc: Quit; F1: Screenshot.", RGB( 255, 255, 0 ) );
				lpDDSBack->ReleaseDC(hdc);
			}
			// Flip the Backbuffer to the primary surface so you can see it...
			Flipscreen();
			return;
	}
} 


//---------------------------------------------------------------------------------------------
// A standard winprocedure. Here you can set keyhandling or whatever...
//---------------------------------------------------------------------------------------------
long FAR PASCAL WindowProc( HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam )
{
	switch( message )
    {
		case WM_ACTIVATEAPP:
			bActive = wParam;
			break;

		case WM_SETCURSOR:
			SetCursor(NULL);
			return TRUE;

		case WM_KEYDOWN:
			switch( wParam )
			{
				case VK_F1:
					SaveJPG( "ScreenShot.jpg", lpDDSPrimary, 80, TRUE );
			        break;
				case VK_ESCAPE:
					PostMessage(hWnd, WM_CLOSE, 0, 0);
					break;
			}
			break;

		case WM_DESTROY:   
			finiObjects();  // Delete the objects...
			PostQuitMessage( 0 ); 
			break;
    }
	
	return DefWindowProc(hWnd, message, wParam, lParam);
}


//---------------------------------------------------------------------------------------------
// Get the surfaces out of memory...
//---------------------------------------------------------------------------------------------
static void finiObjects( void )
{
    if( lpDD != NULL )
    {
        RELEASE( lpDDSPrimary );
        RELEASE( lpDDSJPGNormal );
        RELEASE( lpDDSJPGProgressive );
        RELEASE( lpDDSJPGGrayScale );
        RELEASE( lpDDSPPMNormal );
        RELEASE( lpDDSPGMNormal );
        lpDD->Release();
        lpDD = NULL;
    }
}


//---------------------------------------------------------------------------------------------
// If some Objects were lost, simply restore them...
//---------------------------------------------------------------------------------------------
HRESULT restoreAll( void )
{
	HRESULT     ddrval;

    ddrval = lpDDSPrimary->Restore();
   
	return ddrval;
}


