/****************************************************************************************************************************
**                                                                                                                         **
**                 Copyright (C) 1999 by Punk Productions Electronic Entertainment. All Rights Reserved.                   ** 
**                                                                                                                         **
**        Purpose: Everyone out there should be able to load different Imagefileformats...                                 **
**     Programmer: Nikolaus Brennig, 18, (Nikolaus.Brennig@nol.at)                                                         **
**                 Original ImageLoaderDLL coded by K. Nishita in 1997. I Only used it and created a simple DDloader       **
**                 for it. If you look in the code, its rather easy to use the Loadroutines for standard Win32Apps,        **
**                 since the Data is blitted to a DC, so do what you want with this code.                                  **
**                                                                                                                         **
**                 WARNING: If you want to use this code in a commercial product (game or whatever) you will have          **
**                 to send us ( Nishita and me, nick ) a free and full functional copy of it. Yep, thats business ;-)      **
**                 If you want to use this code only for learning or simply non-commercial use, simply send us an email    **
**                 cause for sure we want to know who out there uses this code...                                          **
**                                                                                                                         **
**                 You can contact us:       Nick: virtualnik@nol.at, http://cust.nol.at/ppee                              **
**                                     K. Nishita: knishita@ibm.net, http://freeware.freeservers.com                       **
**                                                                                                                         **
**                 Ok, thats all for now, so have fun with loading images ;-)                                              **
**                                                                                                                         **
**     Day created: January 12th, 1999                                                                                     **
**                                                                                                                         **
****************************************************************************************************************************/
/* Note:
** If you want to update this code to DirectX6 you will have to change DirectDraw to DirectDraw4,
** the DirectDrawSurface to a DirectDrawSurface4 and the DDSURFACEDESC to a DDSURFACEDESC2 structure.
** And always make sure that the DLL can be found by the program, or better create an errorcheck for it.
** You can do this simply by checking whether hinstLib is NULL or not.
** Yep. and always make sure that your JPEG is NOT PROGRESSIVE encoded, or nothing will be visible.
** The GIFloader only supports the 87 ones. Ok, why we need animated ones in DD???
** Oh yes, this library only works in 16bit or higher mode. Forget it in 8bit.
*/
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <windowsx.h>
#include <ddraw.h>
#include "Images.h"

// Variables...
HINSTANCE      hinstLib; 
FARPROC        ProcAdd; 

// Call this before loading images...
void InitImageViewerDLL()
{
 hinstLib = LoadLibrary( "NViewLib.dll" ); 
 if( hinstLib != NULL ) ProcAdd = GetProcAddress(hinstLib, "NViewLibLoad"); 
}

// call this when finished with loading images...
void ReleaseImageViewerDLL()
{
 FreeLibrary( hinstLib );  
}

// Load a supported Imagefileformat and create a surface to store data there...
LPDIRECTDRAWSURFACE ReadImage( LPDIRECTDRAW pdd, char *Filename, int dx, int dy )
{
 HBITMAP             hbm;
 BITMAP              bm;
 HDC                 hdcImage, hdc;
 DDSURFACEDESC       ddsd;
 LPDIRECTDRAWSURFACE pdds;
 HRESULT             hr;
 int                 W, H;

    // Load the file...
    hbm = (ProcAdd) ( Filename, FALSE );
    // get size of the bitmap
    GetObject(hbm, sizeof(bm), &bm);
    // Check if user wants a specific size...
    if( dx == 0 ) W = bm.bmWidth;  else W = dx;
    if( dy == 0 ) H = bm.bmHeight; else H = dy;
    // Create the Surface...
    ZeroMemory(&ddsd, sizeof(ddsd));
    ddsd.dwSize         = sizeof(ddsd);
    ddsd.dwFlags        = DDSD_CAPS | DDSD_HEIGHT |DDSD_WIDTH;
    ddsd.ddsCaps.dwCaps = DDSCAPS_OFFSCREENPLAIN;
    ddsd.dwWidth        = W;
    ddsd.dwHeight       = H;
    if( pdd->lpVtbl->CreateSurface( pdd, &ddsd, &pdds, NULL ) != DD_OK ) return NULL;
    //  select bitmap into a memoryDC so we can use it.
    hdcImage = CreateCompatibleDC(NULL);
    SelectObject( hdcImage, hbm );
    // get size of the bitmap
    GetObject( hbm, sizeof(bm), &bm );
    // Paint the Bitmap into the surface...
    if( (hr = pdds->lpVtbl->GetDC(pdds, &hdc)) == DD_OK )
    {
      StretchBlt( hdc, 0, 0, W, H, hdcImage, 0, 0, bm.bmWidth, bm.bmHeight, SRCCOPY );
      pdds->lpVtbl->ReleaseDC( pdds, hdc );
    }
    // Free the stuff...
    DeleteDC( hdcImage );
    DeleteObject( hbm );

  return pdds; 
}

// Call this method if a surface was lost...
void RestoreImage( LPDIRECTDRAWSURFACE pdds, char *Filename )
{
 HBITMAP             hbm;
 BITMAP              bm;
 DDSURFACEDESC       ddsd;
 HDC                 hdcImage, hdc;
 HRESULT             hr;

    // make sure the surface is restored...
    pdds->lpVtbl->Restore(pdds);
    // Load the file...
    hbm = (ProcAdd) ( Filename, FALSE );
    // get size of the bitmap
    GetObject(hbm, sizeof(bm), &bm);
    //  select bitmap into a memoryDC so we can use it.
    hdcImage = CreateCompatibleDC(NULL);
    SelectObject( hdcImage, hbm );
    // get size of the bitmap
    GetObject( hbm, sizeof(bm), &bm );
    // Get the size of the surface...
    ddsd.dwSize = sizeof(ddsd);
    ddsd.dwFlags = DDSD_HEIGHT | DDSD_WIDTH;
    pdds->lpVtbl->GetSurfaceDesc(pdds, &ddsd);
    // Paint the Bitmap into the surface...
    if( (hr = pdds->lpVtbl->GetDC(pdds, &hdc)) == DD_OK )
    {
      StretchBlt( hdc, 0, 0, ddsd.dwWidth, ddsd.dwHeight, hdcImage, 0, 0, bm.bmWidth, bm.bmHeight, SRCCOPY );
      pdds->lpVtbl->ReleaseDC( pdds, hdc );
    }
    // Free the stuff...
    DeleteDC( hdcImage );
    DeleteObject( hbm );
}

// Convert a RGB color to a pysical color...
DWORD GetColorMatch( LPDIRECTDRAWSURFACE pdds, COLORREF rgb )
{
  COLORREF       rgbT;
  HDC            hdc;
  DWORD          dw = CLR_INVALID;
  DDSURFACEDESC  ddsd;
  HRESULT        hres;

    // use GDI SetPixel to color match for us...
    if( rgb != CLR_INVALID && pdds->lpVtbl->GetDC(pdds, &hdc) == DD_OK ) 
    {
     rgbT = GetPixel(hdc, 0, 0); SetPixel(hdc, 0, 0, rgb);               
     pdds->lpVtbl->ReleaseDC(pdds, hdc);
    }
    // now lock the surface so we can read back the converted color...
    ddsd.dwSize = sizeof(ddsd);
    while( (hres = pdds->lpVtbl->Lock(pdds, NULL, &ddsd, 0, NULL)) == DDERR_WASSTILLDRAWING );
    if (hres == DD_OK)
    {
     dw = *(DWORD *)ddsd.lpSurface;                    
     if( ddsd.ddpfPixelFormat.dwRGBBitCount < 32) dw &= (1 << ddsd.ddpfPixelFormat.dwRGBBitCount ) - 1;
     pdds->lpVtbl->Unlock(pdds,NULL);
    }
    // now put the color that was there back....
    if (rgb != CLR_INVALID && pdds->lpVtbl->GetDC(pdds, &hdc) == DD_OK)
    {
     SetPixel(hdc, 0, 0, rgbT);
     pdds->lpVtbl->ReleaseDC(pdds,hdc);
    }

  return dw;
}

// MakeColorKey...
HRESULT MakeColorKey( LPDIRECTDRAWSURFACE pdds, COLORREF rgb )
{
 DDCOLORKEY          ddck;
   ddck.dwColorSpaceLowValue  = GetColorMatch(pdds, rgb);
   ddck.dwColorSpaceHighValue = ddck.dwColorSpaceLowValue;
  return pdds->lpVtbl->SetColorKey(pdds, DDCKEY_SRCBLT, &ddck);
}

